#include "debug.h"
#include <malloc.h>
#include "stack.h"

#define STACK_SHRINK_THRESHOLD 30   /* if stack capacity is smaller than this already, don't shrink it */


stack stNew(unsigned int cp) {
  stack st = (stack)malloc(sizeof(struct _stack));
  
  assert (cp > 0);
  st->cpMax = cp;
  st->ap = malloc(st->cpMax * sizeof(void *));
  st->cp = 0;

  return st;
}
  
void FreeStack(stack st) {
  free(st->ap);
  free(st);
}


void PushStack(stack st, void *p) {
  if (st->cp == st->cpMax) {   /* grow */
    st->cpMax *= 2;
    st->ap = realloc(st->ap, st->cpMax * sizeof(void *));
  }
  assert(st->cp < st->cpMax);
  
  st->ap[st->cp++] = p;
}
    

void *pPopStack(stack st) {
  void *p;
  
  assert(st->cp > 0);   /* else, nothing to pop */
  p = st->ap[--st->cp];
  if (st->cp >= STACK_SHRINK_THRESHOLD && st->cp * 3 < st->cpMax) {  /* shrink */
    st->cpMax /= 2;
    st->ap = realloc(st->ap, st->cpMax * sizeof(void *));
  }
  return p;
}


int iStackEntries(stack st) {
  return st->cp;
}


void ClearStackNoResize(stack st) {
  st->cp = 0;
}
